import cfg from "../config.mjs";
import sql from "../sql.mjs";
import lib from "../lib.mjs";
import fs from "fs";
import url from "url";

const allowedMimes = [ "audio", "image", "video" ];
const auth = async (req, res, next) => {
  if(!req.session)
    return res.redirect("/login");
  return next();
};

export default (router, tpl) => {
  router.get(/^\/?(?:tag\/(?<tag>.+?))?(?:user\/(?<user>.+?))?(?:\/(?<mime>image|audio|video))?(?:\/p\/(?<page>\d+))?(?:\/(?<itemid>\d+))?$/, async (req, res) => {
    const user    = req.params.user ?? null;
    const tag     = lib.parseTag(req.params.tag ?? null);
    const mime    = (req.params.mime ?? "");
    const smime   = allowedMimes.includes(mime) ? mime + "/%" : mime === "" ? "%" : "%";
    const page    = +(req.params.page ?? 1);
    const itemid  = +(req.params.itemid ?? null);

    const output = {
      user, tag, mime, smime, page, itemid
    };

    const mode = itemid > 0 ? "item" : "index";
    const modequery = mime == "audio" ? lib.getMode(0) : lib.getMode(req.session.mode ?? 0);
    let data;

    if(user && (await sql("items").where("username", "like", user).count("* as total"))[0].total === 0)
      return res.end("200 - user not found lol");

    if(mode === "item") { // item!
      if(itemid === 404)
        return res.end("404 - lol");
    
      let items = sql("items").select("*").orderBy("items.id", "desc").whereRaw(modequery);
      if(tag) {
        items = items.innerJoin(
          sql("tags")
            .select("tags_assign.item_id", "tags.tag")
            .leftJoin("tags_assign", "tags_assign.tag_id", "tags.id")
            .where("tags.tag", "like", "%"+tag+"%")
            .groupBy("tags_assign.item_id")
            .as("st"),
          "st.item_id", "items.id"
        );
      }
      if(user)
        items = items.andWhere("username", "like", "%" + user + "%");
      if(mime)
        items = items.andWhere("mime", "like", mime + "/%");
      
      items = await items;

      const item = items.findIndex(i => i.id === itemid);
      const actitem = items[item];

      if(!actitem) // sfw-check!
        return res.end("Sorry, this f0ck either doesn't exist (yet) or hasn't been tagged yet, please remain patient, a support assistant will be shortly with you.");
    
      const tags = await lib.getTags(itemid);
      const cheat = items.slice(Math.max(0, item - 3), item + 4).map(i => i.id);
      const link = lib.genLink({ user, tag, mime, act_page: itemid });

      data = {
        user: {
          name: actitem.username,
          channel: actitem.usernetwork == "Telegram" && actitem.userchannel !== "f0ck.me" ? "anonymous" : actitem.userchannel,
          network: actitem.usernetwork
        },
        item: {
          id: actitem.id,
          src: {
            long: actitem.src,
            short: url.parse(actitem.src).hostname,
          },
          thumbnail: `${cfg.websrv.paths.thumbnails}/${actitem.id}.png`,
          coverart: `${cfg.websrv.paths.coverarts}/${actitem.id}.png`,
          dest: `${cfg.websrv.paths.images}/${actitem.dest}`,
          mime: actitem.mime,
          size: lib.formatSize(actitem.size),
          timestamp: lib.timeAgo(new Date(actitem.stamp * 1e3).toISOString()),
          tags: tags
        },
        title: `${actitem.id} - f0ck.me`,
        pagination: {
          end: items[items.length - 1]?.id,
          start: items[0]?.id,
          next: items[item + 1]?.id,
          prev: items[item - 1]?.id,
          page: actitem.id,
          cheat: cheat,
          link: link,
          uff: true
        },
        link: link,
        tmp: output
      };
    }
    else { // page!
      let total, rows;
      
      if(tag) {
        if(tag.match(/sfw/) || tag.length <= 2)
          return res.end("f0ck you!");
        total = (await sql("items")
          .select("items.id", "items.mime", "st.tag")
          .innerJoin(
            sql("tags")
              .select("tags_assign.item_id", "tags.tag")
              .leftJoin("tags_assign", "tags_assign.tag_id", "tags.id")
              .where("tags.tag", "like", "%"+tag+"%")
              .groupBy("tags_assign.item_id")
              .as("st"),
            "st.item_id", "items.id"
          )
          .whereRaw(modequery)
          .count("* as total")
        )[0]?.total;
      }
      else {
        total = (await sql("items")
          .whereRaw(modequery)
          .andWhere("items.mime", "like", smime)
          .andWhere("items.username", "like", user ? user : "%")
          .count("* as total")
        )[0]?.total;
      }

      if(!total || total.length === 0)
        return res.end("no f0cks given lol");

      const pages = +Math.ceil(total / cfg.websrv.eps);
      const act_page = Math.min(pages, page || 1);
      const offset = Math.max(0, (act_page - 1) * cfg.websrv.eps);

      if(tag) {
        rows = await sql("items")
          .select("items.id", "items.mime", "st.tag", "tags_assign.tag_id")
          .joinRaw("left join tags_assign on tags_assign.item_id = items.id and (tags_assign.tag_id = 1 or tags_assign.tag_id = 2)")
          .innerJoin(
            sql("tags")
              .select("tags_assign.item_id", "tags.tag")
              .leftJoin("tags_assign", "tags_assign.tag_id", "tags.id")
              .where("tags.tag", "like", "%"+tag+"%")
              .groupBy("tags_assign.item_id")
              .as("st"),
            "st.item_id", "items.id"
          )
          .whereRaw(modequery)
          .orderBy("items.id", "desc")
          .offset(offset)
          .limit(cfg.websrv.eps);
      }
      else {
        rows = await sql("items")
          .select("items.id", "items.mime", "tags_assign.tag_id")
          .joinRaw("left join tags_assign on tags_assign.item_id = items.id and (tags_assign.tag_id = 1 or tags_assign.tag_id = 2)")
          .whereRaw(modequery)
          .andWhere("items.mime", "like", smime)
          .andWhere("items.username", "like", user ? user : "%")
          .orderBy("items.id", "desc")
          .offset(offset)
          .limit(cfg.websrv.eps);
      }
      
      if(rows.length === 0)
        return res.end("oops");

      rows.forEach(e => {
        if(!fs.existsSync(`public/t/${e.id}.png`))
          fs.copyFileSync("public/s/img/broken.png", `public/t/${e.id}.png`);
      });

      const cheat = [];
      for(let i = Math.max(1, act_page - 3); i <= Math.min(act_page + 3, pages); i++)
        cheat.push(i);

      const link = lib.genLink({ user, tag, mime, act_page });

      data = {
        items: rows,
        pagination: {
          start: 1,
          end: pages,
          prev: (act_page > 1) ? act_page - 1 : null,
          next: (act_page < pages) ? act_page + 1 : null,
          page: act_page,
          cheat: cheat,
          link: link,
          uff: false
        },
        last: rows[rows.length - 1].id,
        link: link,
        tmp: output
      };
    }

    res.reply({ body: tpl.render(mode, data, req) });
  });

  router.get(/^\/(about)$/, (req, res) => {
    res.reply({
      body: tpl.render(req.url.split[0], { tmp: null }, req)
    });
  });

  router.get(/^\/mode\/(\d)/, auth, async (req, res) => {
    const mode = +req.url.split[1];
    let referertmp = req.headers.referer;
    let referer = "";
    
    if(referertmp.match(/f0ck\.me/))
      referer = referertmp.split("/").slice(3).join("/");

    if(cfg.allowedModes[mode]) {
      await sql("user_options")
        .insert({
          user_id: req.session.id,
          mode: mode,
          theme: req.theme ?? "f0ck"
        })
        .onConflict("user_id")
        .merge();
    }
    res.redirect(`/${referer}`);
  });

  router.get(/^\/ranking/, async (req, res) => {
    try {
      const list = await sql('tags_assign')
        .select('prefix', sql.raw('count(distinct tag_id, item_id) count'))
        .groupBy('prefix')
        .orderBy('count', 'desc');

      const stats = await lib.countf0cks();

      res.reply({
        body: tpl.render('ranking', { list, stats, tmp: null }, req)
      });
    } catch(err) {
      res.end(JSON.stringify(err.message));
    }
  });

  return router;
};
